"""Generate a strong password which includes alphabets, numbers, special characters"""

__version__ = '1.2.0'

from .genpasswd import Password
from .functions import NoMultipleChoice, wanted_characters, include_characters, seperator, unwanted_characters, main
from .__main__ import get_argument, gen_Password, main



import random


class NoMultipleChoice(Exception):
    pass


def wanted_characters(characters):
    possibility = ''
    try:
        character = [character for character in characters.split(',')]
    except Exception:
        raise NoMultipleChoice
    for choice in character:
        if choice.lower() == 'alphabets':
            possibility += 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        if choice.lower() == 'lowercase':
            possibility += 'abcdefghijklmnopqrstuvwxyz'
        if choice.lower() == 'uppercase':
            possibility += 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
        if choice.lower() == 'numbers':
            possibility += '0123456789'
        if choice.lower() == 'symbols':
            possibility += str('''!"%&'()*,+-./:;<=>?@[]^_`{|}~”$‘~#\\''')
        if not choice.lower() in ['numbers', 'alphabets', 'uppercase', 'lowercase', 'symbols']:
            for unwanted in choice:
                possibility += unwanted
    return possibility


def include_characters(adding, characters):
    possibility = characters
    try:
        character = [character for character in adding.split(',')]
    except Exception:
        raise NoMultipleChoice
    for choice in character:
        if choice.lower() == 'alphabets':
            for alphabet in "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ":
                if alphabet not in possibility:
                    possibility += alphabet
        if choice.lower() == 'lowercase':
            for alphabet in "abcdefghijklmnopqrstuvwxyz":
                if alphabet not in possibility:
                    possibility += alphabet
        if choice.lower() == 'uppercase':
            for alphabet in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
                if alphabet not in possibility:
                    possibility += alphabet
        if choice.lower() == 'numbers':
            for number in "0123456789":
                if number not in possibility:
                    possibility += number
        if choice.lower() == 'symbols':
            for symbol in str('''!"%&'()*+,-./:;<=>?@[]^_`{|}~”$‘~#\\'''):
                if symbol not in possibility:
                    possibility += symbol
        if not choice.lower() in ['numbers', 'alphabets', 'uppercase', 'lowercase', 'symbols']:
            for temp_var in choice:
                if temp_var not in possibility:
                    possibility += temp_var
    return possibility


def unwanted_characters(unwanted, possibility):
    try:
        choices = [characters for characters in unwanted.split(',')]
    except Exception:
        raise NoMultipleChoice
    if unwanted == ',' or ',,,' in unwanted or ',,' in unwanted:
        possibility = possibility.replace(',', '')
    for choice in choices:
        if choice == 'alphabets':
            for alphabet in "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ":
                possibility = possibility.replace(alphabet, '')
        if choice == 'uppercase':
            for alphabet in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
                possibility = possibility.replace(alphabet, '')
        if choice == 'lowercase':
            for alphabet in "abcdefghijklmnopqrstuvwxyz":
                possibility = possibility.replace(alphabet, '')
        if choice == 'numbers':
            for number in "0123456789":
                possibility = possibility.replace(number, '')
        if choice == 'symbols':
            for symbol in str('''!"%&'()*,+-./:;<=>?@[]^_`{|}~”$‘~#\\'''):
                possibility = possibility.replace(symbol, '')
        if choice not in ['numbers', 'alphabets', 'uppercase', 'lowercase', 'symbols']:
            for temp_var in choice:
                possibility = possibility.replace(temp_var, '')
    return possibility


def seperator(original_pass, sep_length, seperator):
    final_pass = ''
    for i in range(len(original_pass)):
        final_pass += seperator + original_pass[i] if i%sep_length==0 and i!=0 else original_pass[i]
    return final_pass


def main(Length=False, Unwanted=False, Only_char=False, Include=False, Repeat=False, Seperator_length=False, Seperator=False):
    possibility = str(
        '''abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!"%&'()*,+-./:;<=>?@[]^_`{|}~”$‘~#\\''')
    if Unwanted:
        if Only_char:
            possibility = wanted_characters(Only_char)
        if Include:
            possibility = include_characters(Include, possibility)
        possibility = unwanted_characters(Unwanted.lower(), possibility)
    else:
        if Only_char:
            possibility = wanted_characters(Only_char)
        if Include:
            possibility = include_characters(Include, possibility)
    Repeat = 'y' if Repeat is None or Repeat is True else 'n'
    if not Length and not Only_char:
        Length = random.randint(8, 16)
    elif not Length and Only_char:
        return ValueError('[-] Password length must be given.')
    possibility = (possibility * int(Length)) if Repeat.lower() == 'y' else possibility
    if Length and ((int(Length) > len(possibility)) or int(Length) > 72):
        return ValueError('[-] Password length must be less.')
    if Seperator or Seperator_length:
        if Seperator == True:
            if not Seperator_length:
                Seperator_length = 4
            Seperator = "-"
        if not Seperator:
            Seperator = "-"
        if not Seperator_length:
            Seperator_length = 4
        return seperator("".join(random.sample(possibility, int(Length))), Seperator_length, Seperator)
    else:
        return "".join(random.sample(possibility, int(Length)))



from . import functions


class Password:
    def __init__(self, length=False, ignore=False, only=False, include=False, repeat=False, separator=False, separator_length=False, separation=False):
        self.length = length
        self.ignore = ignore
        self.only = only
        self.include = include
        self.repeat = repeat
        self.separator_length = separator_length
        self.separator = separator if separator else separation

    def generate(self):
        return functions.main(Length=self.length, Unwanted=self.ignore, Only_char=self.only, Include=self.include, Repeat=self.repeat, Seperator_length=self.separator_length, Seperator=self.separator)



import optparse
from . import genpasswd


def get_argument():
    parser = optparse.OptionParser(usage='genpasswd [options]', version='genpasswd 1.2.0')
    parser.add_option("-l", "--length", dest="length", type="int", help="To set length to the password")
    parser.add_option("-r", "--repeat", dest="repeat", action='store_true', default=False,
                      help="To repeat the characters in the password")
    parser.add_option("-n", "--no", dest="ignore", help="To ignore unwanted characters to the password")
    parser.add_option("-o", "--only", dest="only", help="To create password only using wanted characters")
    parser.add_option("-i", "--include", dest="include", help="To include characters to the password")
    parser.add_option("-s", "--separator", dest="separator", help="The separator character")
    parser.add_option("-c", "--seplen", dest="separatorlength", type="int", help="The length of characters between separator")
    parser.add_option("--separation", dest="separation", default=False, action="store_true", help="To separate password characters using separator")
    (options, argument) = parser.parse_args()
    return options


def gen_Password(rep, separation, passlen=False, wanted=False, ign=False, inc=False, sep=False, seplen=False):
    arg = genpasswd.Password(length=passlen, only=wanted, ignore=ign, include=inc, repeat=rep, separator=sep, separator_length=seplen, separation=separation)
    passwd = arg.generate()
    return passwd


def main():
    options = get_argument()
    print(f"\n{gen_Password(options.repeat, options.separation, options.length, options.only, options.ignore, options.include, options.separator, options.separatorlength)}")


if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='genpasswd',
    packages=setuptools.find_packages(),
    version='1.2.0',
    license='MIT',
    description='To generate random and strong passwords.',
    author='Gowthaman',
    author_email='rgngowthaman1@gmail.com',
    url='https://github.com/Gowthaman1401/GenPasswd',
    download_url='https://github.com/Gowthaman1401/GenPasswd/releases',
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={'console_scripts': ['genpasswd=genpasswd.__main__:main']
                  },
    python_requires='>=3.6',
)